diff -urw sendmail-8.13.8-orig/sendmail/conf.c sendmail-8.13.8-getmxrr/sendmail/conf.c
--- sendmail-8.13.8-orig/sendmail/conf.c	2006-03-22 17:49:33.000000000 -0500
+++ sendmail-8.13.8-getmxrr/sendmail/conf.c	2007-04-15 14:45:54.000000000 -0400
@@ -6059,6 +6059,14 @@
 	/* Better truncation of list of MX records for dns map. */
 	"_FFR_BESTMX_BETTER_TRUNCATION",
 #endif /* _FFR_BESTMX_BETTER_TRUNCATION */
+#if _FFR_GETMXRR_CANON_CNAME
+	/* If CNAME find when looking for a MX use it for a canonical lookup */
+	"_FFR_GETMXRR_CANON_CNAME",
+# if _FFR_GETMXRR_CANON_CNAME_MX
+	/* chase CNAME's to find MX records */
+	"_FFR_GETMXRR_CANON_CNAME_MX",
+# endif /* _FFR_GETMXRR_CANON_CNAME_MX */
+#endif /* _FFR_GETMXRR_CANON_CNAME */
 #if _FFR_BLOCK_PROXIES
 	/*
 	**  Try to deal with open HTTP proxies that are used to send spam
diff -urw sendmail-8.13.8-orig/sendmail/domain.c sendmail-8.13.8-getmxrr/sendmail/domain.c
--- sendmail-8.13.8-orig/sendmail/domain.c	2006-04-17 20:00:34.000000000 -0400
+++ sendmail-8.13.8-getmxrr/sendmail/domain.c	2006-11-03 10:12:50.000000000 -0500
@@ -199,6 +199,13 @@
 **		MXHostBuf[MXHOSTBUFSIZE], so the data needs to be copied,
 **		if it must be preserved across calls to this function.
 */
+#if _FFR_GETMXRR_CANON_CNAME_MX
+static struct getmxrr_recdesc 
+{
+	char *name;
+	int level;
+} Getmxrr_Recdesc;
+#endif /* _FFR_GETMXRR_CANON_CNAME_MX */
 
 int
 getmxrr(host, mxhosts, mxprefs, droplocalhost, rcode, tryfallback, pttl)
@@ -269,6 +276,16 @@
 	else
 		resfunc = res_search;
 
+#if _FFR_GETMXRR_CANON_CNAME_MX
+	if (Getmxrr_Recdesc.name && strcmp(Getmxrr_Recdesc.name, host) == 0)
+	{
+		if (Getmxrr_Recdesc.level++ >= MAXCNAMEDEPTH)
+			goto punt;
+	}
+	else
+		Getmxrr_Recdesc.level = 0;
+#endif /* _FFR_GETMXRR_CANON_CNAME_MX */
+
 	errno = 0;
 	n = (*resfunc)(host, C_IN, T_MX, (unsigned char *) &answer,
 		       sizeof(answer));
@@ -355,6 +372,27 @@
 		GETSHORT(n, cp);	/* rdlength */
 		if (type != T_MX)
 		{
+# if _FFR_GETMXRR_CANON_CNAME
+			if (type == T_CNAME)
+			{
+#  if _FFR_GETMXRR_CANON_CNAME_MX
+				if ((n = dn_expand((unsigned char *)&answer, eom, cp,
+						   (RES_UNC_T) bp, buflen)) < 0)
+					break;
+				cp += n;
+				n = strlen(bp);
+				if (tTd(8, 5))
+					sm_dprintf("Found a CNAME, checking MX for %s\n", bp);
+				Getmxrr_Recdesc.name = bp;
+				return getmxrr(bp, mxhosts, mxprefs, droplocalhost, rcode, tryfallback, pttl);
+#  endif /* _FFR_GETMXRR_CANON_CNAME_MX */
+				if (tTd(8, 5))
+					sm_dprintf("Found a CNAME, will try canonical name"
+					" if there are no MX's\n");
+				trycanon = true;
+			}
+			else
+# endif /* _FFR_GETMXRR_CANON_CNAME */
 			if (tTd(8, 8) || _res.options & RES_DEBUG)
 				sm_dprintf("unexpected answer type %d, size %d\n",
 					type, n);
